window.theme = {};

// Theme Common Functions
window.theme.fn = {

    getOptions: function ( opts ) {

        if ( typeof ( opts ) == 'object' ) {

            return opts;

        } else if ( typeof ( opts ) == 'string' ) {

            try {
                return JSON.parse( opts.replace( /'/g, '"' )
                    .replace( ';', '' ) );
            } catch ( e ) {
                return {};
            }

        } else {

            return {};

        }

    }

};

// Scroll to Top
( function ( theme, $ ) {

    theme = theme || {};

    $.extend( theme, {

        ScrollToTop: {

            defaults: {
                wrapper: $( 'body' ),
                offset: 150,
                buttonClass: 'scroll-to-top',
                iconClass: 'fas fa-chevron-up',
                delay: 1000,
                visibleMobile: false,
                label: false,
                easing: 'easeOutBack'
            },

            initialize: function ( opts ) {
                initialized = true;

                // Don't initialize if the page has Section Scroll
                if ( $( 'body[data-section-scroll]' )
                    .get( 0 ) ) {
                    return;
                }

                this
                    .setOptions( opts )
                    .build()
                    .events();

                return this;
            },

            setOptions: function ( opts ) {
                this.options = $.extend( true, {}, this.defaults, opts );

                return this;
            },

            build: function () {
                var self = this,
                    $el;

                // Base HTML Markup
                $el = $( '<a />' )
                    .addClass( self.options.buttonClass )
                    .attr( {
                        'href': '#',
                    } )
                    .append(
                        $( '<i />' )
                        .addClass( self.options.iconClass )
                    );

                // Visible Mobile
                if ( !self.options.visibleMobile ) {
                    $el.addClass( 'hidden-mobile' );
                }

                // Label
                if ( self.options.label ) {
                    $el.append(
                        $( '<span />' )
                        .html( self.options.label )
                    );
                }

                this.options.wrapper.append( $el );

                this.$el = $el;

                return this;
            },

            events: function () {
                var self = this,
                    _isScrolling = false;

                // Click Element Action
                self.$el.on( 'click', function ( e ) {
                    e.preventDefault();
                    $( 'body, html' )
                        .animate( {
                            scrollTop: 0
                        }, self.options.delay, self.options.easing );
                    return false;
                } );

                // Show/Hide Button on Window Scroll event.
                $( window )
                    .scroll( function () {

                        if ( !_isScrolling ) {

                            _isScrolling = true;

                            if ( $( window )
                                .scrollTop() > self.options.offset ) {

                                self.$el.stop( true, true )
                                    .addClass( 'visible' );
                                _isScrolling = false;

                            } else {

                                self.$el.stop( true, true )
                                    .removeClass( 'visible' );
                                _isScrolling = false;

                            }

                        }

                    } );

                return this;
            }

        }

    } );

} )
.apply( this, [ window.theme, jQuery ] );

// Section Scroll
( function ( theme, $ ) {

    theme = theme || {};

    var instanceName = '__sectionScroll';

    var SectionScroll = function ( $el, opts ) {
        return this.initialize( $el, opts );
    };

    SectionScroll.defaults = {
        targetClass: '.section',
        dotsNav: true,
        changeHeaderLogo: false,
        headerLogoDark: 'images/logo-lite.png',
        headerLogoLight: 'images/logo-lite-dark.png'
    };

    SectionScroll.prototype = {
        initialize: function ( $el, opts ) {
            if ( $el.data( instanceName ) ) {
                return this;
            }

            this.$el = $el;

            this
                .setData()
                .setOptions( opts )
                .build()
                .events();

            return this;
        },

        setData: function () {
            this.$el.data( instanceName, this );

            return this;
        },

        setOptions: function ( opts ) {
            this.options = $.extend( true, {}, SectionScroll.defaults, opts, {
                wrapper: this.$el
            } );

            return this;
        },

        build: function () {
            var self = this,
                $el = this.options.wrapper;

            // Check type of header and change the target for header (by change header color purpose)
            if ( $( 'html' )
                .hasClass( 'side-header-overlay-full-screen' ) ) {
                self.$header = $( '.sticky-wrapper' );
            } else {
                self.$header = $( '#header' );
            }

            // Turn the section full height or not depeding on the content size
            self.updateSectionsHeight();

            // Wrap all sections in a section wrapper
            $( this.options.targetClass )
                .wrap( '<div class="section-wrapper"></div>' );

            // Set the section wrapper height
            $( '.section-wrapper' )
                .each( function () {
                    $( this )
                        .height( $( this )
                            .find( '.section-scroll' )
                            .outerHeight() );
                } );

            // Add active class to the first section on page load
            $( '.section-wrapper' )
                .first()
                .addClass( 'active' );

            var flag = false,
                scrollableFlag = false,
                touchDirection = '',
                touchstartY = 0,
                touchendY = 0;

            $( window )
                .on( 'touchstart', function ( event ) {
                    touchstartY = event.changedTouches[ 0 ].screenY;
                } );

            var wheelEvent = 'onwheel' in document ? 'wheel' : document.onmousewheel !== undefined ? 'mousewheel' : 'DOMMouseScroll';
            if ( $( window )
                .width() < 992 && $( 'html' )
                .hasClass( 'touch' ) ) {
                wheelEvent = 'onwheel' in document ? 'wheel touchend' : document.onmousewheel !== undefined ? 'mousewheel touchend' : 'DOMMouseScroll touchend';
            }

            $( window )
                .on( wheelEvent, function ( e ) {
                    if ( $( window )
                        .width() < 992 && $( 'html' )
                        .hasClass( 'touch' ) ) {
                        if ( $( e.target )
                            .closest( '.section-scroll-dots-navigation' )
                            .get( 0 ) || $( e.target )
                            .closest( '.header-body' )
                            .get( 0 ) || $( e.target )
                            .closest( '.owl-carousel' )
                            .get( 0 ) ) {
                            return;
                        }
                    }

                    // Side Header Overlay Full Screen
                    if ( $( 'html.side-header-overlay-full-screen.side-header-hide' )
                        .get( 0 ) ) {
                        return;
                    }

                    var wheelDirection = e.originalEvent.wheelDelta == undefined ? e.originalEvent.deltaY > 0 : e.originalEvent.wheelDelta < 0;
                    if ( $( window )
                        .width() < 992 && $( 'html' )
                        .hasClass( 'touch' ) ) {
                        touchendY = event.changedTouches[ 0 ].screenY;

                        if ( touchendY <= touchstartY ) {
                            touchDirection = 'up';
                        }

                        if ( touchendY >= touchstartY ) {
                            touchDirection = 'down';
                        }

                        if ( touchendY == touchstartY ) {
                            return;
                        }
                    }

                    var $currentSection = $( '.section-wrapper' )
                        .eq( self.getCurrentIndex() )
                        .find( '.section-scroll' ),
                        $nextSection = self.getNextSection( wheelDirection, touchDirection ),
                        nextSectionOffsetTop;

                    // If is the last section, then change the offsetTop value
                    if ( self.getCurrentIndex() == $( '.section-wrapper' )
                        .length - 1 ) {
                        nextSectionOffsetTop = $( document )
                            .height();
                    } else {
                        nextSectionOffsetTop = $nextSection.offset()
                            .top;
                    }

                    if ( $( window )
                        .width() < 992 && $( 'html' )
                        .hasClass( 'touch' ) ) {
                        setTimeout( function () {
                            if ( $( '.section-wrapper' )
                                .eq( self.getCurrentIndex() )
                                .find( '.section-scroll' )
                                .hasClass( 'section-scroll-scrollable' ) ) {
                                $( 'html' )
                                    .removeClass( 'overflow-hidden' );
                            } else {
                                $( 'html' )
                                    .addClass( 'overflow-hidden' );
                            }
                        }, 1200 );
                    }

                    // For non full height sections
                    if ( $currentSection.hasClass( 'section-scroll-scrollable' ) ) {
                        if ( !flag && !scrollableFlag ) {

                            // Scroll Direction
                            if ( wheelDirection || touchDirection == 'up' ) {
                                if ( ( $( window )
                                        .scrollTop() + $( window )
                                        .height() ) >= nextSectionOffsetTop ) {
                                    flag = true;
                                    setTimeout( function () {
                                        $( window )
                                            .trigger( 'section.scroll.change.header.color' );

                                        setTimeout( function () {
                                            flag = false;
                                        }, 500 );
                                    }, 1000 );

                                    if ( self.getCurrentIndex() == ( $( '.section-wrapper' )
                                            .length - 1 ) ) {
                                        return false;
                                    }

                                    // Move to the next section
                                    self.moveTo( $currentSection.offset()
                                        .top + $currentSection.outerHeight() );

                                    // Change Section Active Class
                                    self.changeSectionActiveState( $nextSection );

                                    self.$header.css( {
                                        opacity: 0,
                                        transition: 'ease opacity 500ms'
                                    } );
                                }

                                if ( !$( 'html' )
                                    .hasClass( 'touch' ) ) {
                                    for ( var i = 1; i < 100; i++ ) {
                                        $( 'body, html' )
                                            .scrollTop( $( window )
                                                .scrollTop() + 1 );

                                        if ( ( $( window )
                                                .scrollTop() + $( window )
                                                .height() ) >= nextSectionOffsetTop ) {
                                            scrollableFlag = true;
                                            setTimeout( function () {
                                                $( window )
                                                    .trigger( 'section.scroll.change.header.color' );
                                                scrollableFlag = false;
                                            }, 500 );
                                            break;
                                        }
                                    }
                                }
                            } else {
                                if ( $( window )
                                    .scrollTop() <= $currentSection.offset()
                                    .top ) {
                                    flag = true;
                                    setTimeout( function () {
                                        $( window )
                                            .trigger( 'section.scroll.change.header.color' );

                                        setTimeout( function () {
                                            flag = false;
                                        }, 500 );
                                    }, 1000 );

                                    if ( self.getCurrentIndex() == 0 ) {
                                        return false;
                                    }

                                    // Move to the next section
                                    self.moveTo( $currentSection.offset()
                                        .top - $( window )
                                        .height() );

                                    // Change Section Active Class
                                    self.changeSectionActiveState( $nextSection );

                                    self.$header.css( {
                                        opacity: 0,
                                        transition: 'ease opacity 500ms'
                                    } );
                                }

                                if ( !$( 'html' )
                                    .hasClass( 'touch' ) ) {
                                    for ( var i = 1; i < 100; i++ ) {
                                        $( 'body, html' )
                                            .scrollTop( $( window )
                                                .scrollTop() - 1 );

                                        if ( $( window )
                                            .scrollTop() <= $currentSection.offset()
                                            .top ) {
                                            scrollableFlag = true;
                                            setTimeout( function () {
                                                $( window )
                                                    .trigger( 'section.scroll.change.header.color' );
                                                scrollableFlag = false;
                                            }, 500 );
                                            break;
                                        }
                                    }
                                }
                            }

                            // Change Dots Active Class
                            self.changeDotsActiveState();

                            return;

                        }
                    }

                    // For full height sections
                    if ( !flag && !scrollableFlag ) {
                        if ( wheelDirection || touchDirection == 'up' ) {
                            if ( self.getCurrentIndex() == ( $( '.section-wrapper' )
                                    .length - 1 ) ) {
                                return false;
                            }

                            // Change Section Active Class
                            self.changeSectionActiveState( $nextSection );

                            setTimeout( function () {
                                // Move to the next section
                                self.moveTo( $nextSection.offset()
                                    .top );

                            }, 150 );
                        } else {
                            if ( self.getCurrentIndex() == 0 ) {
                                return false;
                            }

                            // Change Section Active Class
                            self.changeSectionActiveState( $nextSection );

                            if ( $nextSection.height() > $( window )
                                .height() ) {
                                // Move to the next section
                                self.moveTo( $currentSection.offset()
                                    .top - $( window )
                                    .height() );
                            } else {
                                setTimeout( function () {
                                    // Move to the next section
                                    self.moveTo( $nextSection.offset()
                                        .top );

                                }, 150 );
                            }
                        }

                        // Change Dots Active Class
                        self.changeDotsActiveState();

                        self.$header.css( {
                            opacity: 0,
                            transition: 'ease opacity 500ms'
                        } );

                        // Style next section
                        $nextSection.css( {
                            position: 'relative',
                            opacity: 1,
                            'z-index': 1,
                            transform: 'translate3d(0,0,0) scale(1)'
                        } );

                        // Style previous section
                        $currentSection.css( {
                            position: 'fixed',
                            width: '100%',
                            top: 0,
                            left: 0,
                            opacity: 0,
                            'z-index': 0,
                            transform: 'translate3d(0,0,-10px) scale(0.7)',
                            transition: 'ease transform 600ms, ease opacity 600ms',
                        } );

                        setTimeout( function () {
                            $currentSection.css( {
                                position: 'relative',
                                opacity: 1,
                                transform: 'translate3d(0,0,-10px) scale(1)'
                            } );

                            $( window )
                                .trigger( 'section.scroll.change.header.color' );

                            setTimeout( function () {
                                flag = false;
                            }, 500 );
                        }, 1000 );

                        flag = true;

                    }

                    return;
                } );

            // Dots Navigation
            if ( this.options.dotsNav ) {
                self.dotsNavigation();
            }

            // First Load
            setTimeout( function () {
                if ( $( window.location.hash )
                    .get( 0 ) ) {
                    self.moveTo( $( window.location.hash )
                        .parent()
                        .offset()
                        .top );

                    self.changeSectionActiveState( $( window.location.hash ) );

                    // Change Dots Active Class
                    self.changeDotsActiveState();

                    self.updateHash( true );
                } else {
                    var hash = window.location.hash,
                        index = hash.replace( '#', '' );

                    if ( !hash ) {
                        index = 1;
                    }

                    self.moveTo( $( '.section-wrapper' )
                        .eq( index - 1 )
                        .offset()
                        .top );

                    self.changeSectionActiveState( $( '.section-wrapper' )
                        .eq( index - 1 )
                        .find( '.section-scroll' ) );

                    // Change Dots Active Class
                    self.changeDotsActiveState();

                    self.updateHash( true );
                }

                $( window )
                    .trigger( 'section.scroll.ready' );
            }, 500 );

            return this;
        },

        updateSectionsHeight: function () {
            var self = this;

            $( '.section-scroll' )
                .css( {
                    height: ''
                } );

            $( '.section-scroll' )
                .each( function () {
                    if ( $( this )
                        .outerHeight() < ( $( window )
                            .height() + 3 ) ) {
                        $( this )
                            .css( {
                                height: '100vh'
                            } );
                    } else {
                        $( this )
                            .addClass( 'section-scroll-scrollable' );
                    }
                } );

            // Set the section wrapper height
            $( '.section-wrapper' )
                .each( function () {
                    $( this )
                        .height( $( this )
                            .find( '.section-scroll' )
                            .outerHeight() );
                } );

            return this;
        },

        updateHash: function ( first_load ) {
            var self = this;

            if ( !window.location.hash ) {
                window.location.hash = 1;
            } else {
                if ( !first_load ) {
                    var $section = $( '.section-wrapper' )
                        .eq( self.getCurrentIndex() )
                        .find( '.section-scroll' ),
                        section_id = $section.attr( 'id' ) ? $section.attr( 'id' ) : $section.parent()
                        .index() + 1;

                    window.location.hash = section_id;
                }
            }

            return this;
        },

        getCurrentIndex: function () {
            var self = this,
                currentIndex = 0;

            currentIndex = $( '.section-wrapper.active' )
                .index();

            return currentIndex;
        },

        moveTo: function ( $scrollTopValue, first_load ) {
            var self = this;

            $( 'body, html' )
                .animate( {
                    scrollTop: $scrollTopValue
                }, 1000, 'easeOutQuint' );

            setTimeout( function () {
                self.updateHash();
            }, 500 );

            return this;
        },

        getNextSection: function ( wheelDirection, touchDirection ) {
            var self = this,
                $nextSection = '';

            // Scroll Direction
            if ( wheelDirection || touchDirection == 'up' ) {
                $nextSection = $( '.section-wrapper' )
                    .eq( self.getCurrentIndex() + 1 )
                    .find( '.section-scroll' );
            } else {
                $nextSection = $( '.section-wrapper' )
                    .eq( self.getCurrentIndex() - 1 )
                    .find( '.section-scroll' );
            }

            return $nextSection;
        },

        changeSectionActiveState: function ( $nextSection ) {
            var self = this;

            $( '.section-wrapper' )
                .removeClass( 'active' );
            $nextSection.parent()
                .addClass( 'active' );

            return this;
        },

        changeDotsActiveState: function () {
            var self = this;

            $( '.section-scroll-dots-navigation > ul > li' )
                .removeClass( 'active' );
            $( '.section-scroll-dots-navigation > ul > li' )
                .eq( self.getCurrentIndex() )
                .addClass( 'active' );

            return this;
        },

        dotsNavigation: function () {
            var self = this;

            var dotsNav = $( '<div class="section-scroll-dots-navigation"><ul class="list list-unstyled"></ul></div>' ),
                currentSectionIndex = self.getCurrentIndex();

            if ( self.options.dotsClass ) {
                dotsNav.addClass( self.options.dotsClass );
            }

            for ( var i = 0; i < $( '.section-scroll' )
                .length; i++ ) {
                var title = $( '.section-wrapper' )
                    .eq( i )
                    .find( '.section-scroll' )
                    .data( 'section-scroll-title' );

                dotsNav.find( '> ul' )
                    .append( '<li' + ( ( currentSectionIndex == i ) ? ' class="active"' : '' ) + '><a href="#' + i + '" data-nav-id="' + i + '"><span>' + title + '</span></a></li>' );
            }

            $( '.body' )
                .append( dotsNav );

            dotsNav.find( 'a[data-nav-id]' )
                .on( 'click touchstart', function ( e ) {
                    e.preventDefault();
                    var $this = $( this );

                    $( '.section-scroll' )
                        .css( {
                            opacity: 0,
                            transition: 'ease opacity 300ms'
                        } );

                    self.$header.css( {
                        opacity: 0,
                        transition: 'ease opacity 500ms'
                    } );

                    setTimeout( function () {
                        self.moveTo( $( '.section-wrapper' )
                            .eq( $this.data( 'nav-id' ) )
                            .offset()
                            .top );

                        $( '.section-wrapper' )
                            .removeClass( 'active' );
                        $( '.section-wrapper' )
                            .eq( $this.data( 'nav-id' ) )
                            .addClass( 'active' );

                        $( '.section-wrapper' )
                            .eq( self.getCurrentIndex() )
                            .find( '.section-scroll' )
                            .css( {
                                opacity: 1
                            } );

                        setTimeout( function () {
                            $( '.section-scroll' )
                                .css( {
                                    opacity: 1
                                } );

                            $( window )
                                .trigger( 'section.scroll.change.header.color' );
                        }, 500 );

                        self.changeDotsActiveState();
                    }, 500 );
                } );

            return this;
        },

        events: function () {
            var self = this;

            $( window )
                .on( 'section.scroll.ready', function () {
                    $( window )
                        .scrollTop( 0 );
                } );

            $( window )
                .on( 'section.scroll.change.header.color', function () {
                    var headerColor = $( '.section-wrapper' )
                        .eq( self.getCurrentIndex() )
                        .find( '.section-scroll' )
                        .data( 'section-scroll-header-color' );

                    $( '#header .header-nav' )
                        .removeClass( 'header-nav-light-text header-nav-dark-text' )
                        .addClass( 'header-nav-' + headerColor + '-text' );
                    $( '#header .header-nav-features' )
                        .removeClass( 'header-nav-features-dark header-nav-features-light' )
                        .addClass( 'header-nav-features-' + headerColor );
                    $( '#header .header-social-icons' )
                        .removeClass( 'social-icons-icon-dark social-icons-icon-light' )
                        .addClass( 'social-icons-icon-' + headerColor );

                    // Change Logo
                    if ( self.options.changeHeaderLogo && headerColor != undefined ) {
                        if ( headerColor == 'light' ) {
                            $( '#header .header-logo img' )
                                .attr( 'src', self.options.headerLogoLight );
                        } else if ( headerColor == 'dark' ) {
                            $( '#header .header-logo img' )
                                .attr( 'src', self.options.headerLogoDark );
                        }
                    }

                    self.$header.css( {
                        opacity: 1
                    } );
                } );

            $( document )
                .ready( function () {
                    $( window )
                        .afterResize( function () {
                            self.updateSectionsHeight();
                        } );
                } );

            return this;
        }
    };

    // expose to scope
    $.extend( theme, {
        SectionScroll: SectionScroll
    } );

    // jquery plugin
    $.fn.themeSectionScroll = function ( opts ) {
        return this.map( function () {
            var $this = $( this );

            if ( $this.data( instanceName ) ) {
                return $this.data( instanceName );
            } else {
                return new SectionScroll( $this, opts );
            }

        } );
    };

} )
.apply( this, [ window.theme, jQuery ] );

// Nav
( function ( theme, $ ) {

    theme = theme || {};

    var initialized = false;

    $.extend( theme, {

        Nav: {

            defaults: {
                wrapper: $( '#mainNav' ),
                scrollDelay: 600,
                scrollAnimation: 'easeOutQuad'
            },

            initialize: function ( $wrapper, opts ) {
                if ( initialized ) {
                    return this;
                }

                initialized = true;
                this.$wrapper = ( $wrapper || this.defaults.wrapper );

                this
                    .setOptions( opts )
                    .build()
                    .events();

                return this;
            },

            setOptions: function ( opts ) {
                this.options = $.extend( true, {}, this.defaults, opts, theme.fn.getOptions( this.$wrapper.data( 'plugin-options' ) ) );

                return this;
            },

            build: function () {
                var self = this,
                    $html = $( 'html' ),
                    $header = $( '#header' ),
                    $headerNavMain = $( '#header .header-nav-main' ),
                    thumbInfoPreview;

                // Preview Thumbs
                self.$wrapper.find( 'a[data-thumb-preview]' )
                    .each( function () {
                        thumbInfoPreview = $( '<span />' )
                            .addClass( 'thumb-info thumb-info-preview' )
                            .append( $( '<span />' )
                                .addClass( 'thumb-info-wrapper' )
                                .append( $( '<span />' )
                                    .addClass( 'thumb-info-image' )
                                    .css( 'background-image', 'url(' + $( this )
                                        .data( 'thumb-preview' ) + ')' )
                                )
                            );

                        $( this )
                            .append( thumbInfoPreview );
                    } );

                // Side Header / Side Header Hamburguer Sidebar (Reverse Dropdown)
                if ( $html.hasClass( 'side-header' ) || $html.hasClass( 'side-header-hamburguer-sidebar' ) ) {

                    // Side Header Right / Side Header Hamburguer Sidebar Right
                    if ( $html.hasClass( 'side-header-right' ) || $html.hasClass( 'side-header-hamburguer-sidebar-right' ) ) {
                        if ( !$html.hasClass( 'side-header-right-no-reverse' ) ) {
                            $header.find( '.dropdown-submenu' )
                                .addClass( 'dropdown-reverse' );
                        }
                    }

                } else {

                    // Reverse
                    self.checkReverse = function () {
                        self.$wrapper.find( '.dropdown, .dropdown-submenu' )
                            .removeClass( 'dropdown-reverse' );

                        self.$wrapper.find( '.dropdown:not(.manual):not(.dropdown-mega), .dropdown-submenu:not(.manual)' )
                            .each( function () {
                                if ( !$( this )
                                    .find( '.dropdown-menu' )
                                    .visible( false, true, 'horizontal' ) ) {
                                    $( this )
                                        .addClass( 'dropdown-reverse' );
                                }
                            } );
                    };

                    self.checkReverse();

                    $( window )
                        .on( 'resize', function () {
                            self.checkReverse();
                        } );

                }

                // Clone Items
                if ( $headerNavMain.hasClass( 'header-nav-main-clone-items' ) ) {

                    $headerNavMain.find( 'nav > ul > li > a' )
                        .each( function () {
                            var parent = $( this )
                                .parent(),
                                clone = $( this )
                                .clone(),
                                clone2 = $( this )
                                .clone(),
                                wrapper = $( '<span class="wrapper-items-cloned"></span>' );

                            // Config Classes
                            $( this )
                                .addClass( 'item-original' );
                            clone2.addClass( 'item-two' );

                            // Insert on DOM
                            parent.prepend( wrapper );
                            wrapper.append( clone )
                                .append( clone2 );
                        } );

                }

                // Floating
                if ( $( '#header.header-floating-icons' )
                    .get( 0 ) && $( window )
                    .width() > 991 ) {

                    var menuFloatingAnim = {
                        $menuFloating: $( '#header.header-floating-icons .header-container > .header-row' ),

                        build: function () {
                            var self = this;

                            self.init();
                        },
                        init: function () {
                            var self = this,
                                divisor = 0;

                            $( window )
                                .scroll( function () {
                                    var scrollPercent = 100 * $( window )
                                        .scrollTop() / ( $( document )
                                            .height() - $( window )
                                            .height() ),
                                        st = $( this )
                                        .scrollTop();

                                    divisor = $( document )
                                        .height() / $( window )
                                        .height();

                                    self.$menuFloating.find( '.header-column > .header-row' )
                                        .css( {
                                            transform: 'translateY( calc(' + scrollPercent + 'vh - ' + st / divisor + 'px) )'
                                        } );
                                } );
                        }
                    };

                    menuFloatingAnim.build();

                }

                // Slide
                if ( $( '.header-nav-links-vertical-slide' )
                    .get( 0 ) ) {
                    var slideNavigation = {
                        $mainNav: $( '#mainNav' ),
                        $mainNavItem: $( '#mainNav li' ),

                        build: function () {
                            var self = this;

                            self.menuNav();
                        },
                        menuNav: function () {
                            var self = this;

                            self.$mainNavItem.on( 'click', function ( e ) {
                                var currentMenuItem = $( this ),
                                    currentMenu = $( this )
                                    .parent(),
                                    nextMenu = $( this )
                                    .find( 'ul' )
                                    .first(),
                                    prevMenu = $( this )
                                    .closest( '.next-menu' ),
                                    isSubMenu = currentMenuItem.hasClass( 'dropdown' ) || currentMenuItem.hasClass( 'dropdown-submenu' ),
                                    isBack = currentMenuItem.hasClass( 'back-button' ),
                                    nextMenuHeightDiff = ( ( nextMenu.find( '> li' )
                                        .length * nextMenu.find( '> li' )
                                        .outerHeight() ) - nextMenu.outerHeight() ),
                                    prevMenuHeightDiff = ( ( prevMenu.find( '> li' )
                                        .length * prevMenu.find( '> li' )
                                        .outerHeight() ) - prevMenu.outerHeight() );

                                if ( isSubMenu ) {
                                    currentMenu.addClass( 'next-menu' );
                                    nextMenu.addClass( 'visible' );
                                    currentMenu.css( {
                                        overflow: 'visible',
                                        'overflow-y': 'visible'
                                    } );

                                    if ( nextMenuHeightDiff > 0 ) {
                                        nextMenu.css( {
                                            overflow: 'hidden',
                                            'overflow-y': 'scroll'
                                        } );
                                    }

                                    for ( i = 0; i < nextMenu.find( '> li' )
                                        .length; i++ ) {
                                        if ( nextMenu.outerHeight() < ( $( '.header-row-side-header' )
                                                .outerHeight() - 100 ) ) {
                                            nextMenu.css( {
                                                height: nextMenu.outerHeight() + nextMenu.find( '> li' )
                                                    .outerHeight()
                                            } );
                                        }
                                    }

                                    nextMenu.css( {
                                        'padding-top': nextMenuHeightDiff + 'px'
                                    } );
                                }

                                if ( isBack ) {
                                    currentMenu.parent()
                                        .parent()
                                        .removeClass( 'next-menu' );
                                    currentMenu.removeClass( 'visible' );

                                    if ( prevMenuHeightDiff > 0 ) {
                                        prevMenu.css( {
                                            overflow: 'hidden',
                                            'overflow-y': 'scroll'
                                        } );
                                    }
                                }

                                e.stopPropagation();
                            } );
                        }
                    };

                    $( window )
                        .trigger( 'resize' );

                    if ( $( window )
                        .width() > 991 ) {
                        slideNavigation.build();
                    }

                    $( document )
                        .ready( function () {
                            $( window )
                                .afterResize( function () {
                                    if ( $( window )
                                        .width() > 991 ) {
                                        slideNavigation.build();
                                    }
                                } );
                        } );
                }

                // Header Nav Main Mobile Dark
                if ( $( '.header-nav-main-mobile-dark' )
                    .get( 0 ) ) {
                    $( '#header:not(.header-transparent-dark-bottom-border):not(.header-transparent-light-bottom-border)' )
                        .addClass( 'header-no-border-bottom' );
                }

                return this;
            },

            events: function () {
                var self = this,
                    $html = $( 'html' ),
                    $header = $( '#header' ),
                    $window = $( window ),
                    headerBodyHeight = $( '.header-body' )
                    .outerHeight();

                $header.find( 'a[href="#"]' )
                    .on( 'click', function ( e ) {
                        e.preventDefault();
                    } );

                // Mobile Arrows
                $header.find( '.dropdown-toggle, .dropdown-submenu > a' )
                    .append( '<i class="fas fa-chevron-down"></i>' );

                $header.find( '.dropdown-toggle[href="#"], .dropdown-submenu a[href="#"], .dropdown-toggle[href!="#"] .fa-chevron-down, .dropdown-submenu a[href!="#"] .fa-chevron-down' )
                    .on( 'click', function ( e ) {
                        e.preventDefault();
                        if ( $window.width() < 992 ) {
                            $( this )
                                .closest( 'li' )
                                .toggleClass( 'open' );

                            // Adjust Header Body Height
                            var height = ( $header.hasClass( 'header-effect-shrink' ) && $html.hasClass( 'sticky-header-active' ) ) ? theme.StickyHeader.options.stickyHeaderContainerHeight : headerBodyHeight;
                            $( '.header-body' )
                                .animate( {
                                    height: ( $( '.header-nav-main nav' )
                                        .outerHeight( true ) + height ) + 10
                                }, 0 );
                        }
                    } );

                // Add Open Class
                $header.find( '.header-nav-click-to-open .dropdown-toggle[href="#"], .header-nav-click-to-open .dropdown-submenu a[href="#"]' )
                    .on( 'click', function ( e ) {
                        e.preventDefault();
                        if ( $window.width() > 991 ) {

                            if ( !$( this )
                                .closest( 'li' )
                                .hasClass( 'open' ) ) {

                                var $li = $( this )
                                    .closest( 'li' ),
                                    isSub = false;

                                if ( $( this )
                                    .parent()
                                    .hasClass( 'dropdown-submenu' ) ) {
                                    isSub = true;
                                }

                                $( this )
                                    .closest( '.dropdown-menu' )
                                    .find( '.dropdown-submenu.open' )
                                    .removeClass( 'open' );
                                $( this )
                                    .parent( '.dropdown' )
                                    .parent()
                                    .find( '.dropdown.open' )
                                    .removeClass( 'open' );

                                if ( !isSub ) {
                                    $( this )
                                        .parent()
                                        .find( '.dropdown-submenu.open' )
                                        .removeClass( 'open' );
                                }

                                $li.addClass( 'open' );

                                $( document )
                                    .off( 'click.nav-click-to-open' )
                                    .on( 'click.nav-click-to-open', function ( e ) {
                                        if ( !$li.is( e.target ) && $li.has( e.target )
                                            .length === 0 ) {
                                            $li.removeClass( 'open' );
                                            $li.parents( '.open' )
                                                .removeClass( 'open' );
                                        }
                                    } );

                            } else {
                                $( this )
                                    .closest( 'li' )
                                    .removeClass( 'open' );
                            }

                            $window.trigger( 'resize' );
                        }
                    } );

                // Collapse Nav
                $header.find( '[data-collapse-nav]' )
                    .on( 'click', function ( e ) {
                        $( this )
                            .parents( '.collapse' )
                            .removeClass( 'show' );
                    } );

                // Top Features
                $header.find( '.header-nav-features-toggle' )
                    .on( 'click', function ( e ) {
                        e.preventDefault();

                        var $toggleParent = $( this )
                            .parent();

                        if ( !$( this )
                            .siblings( '.header-nav-features-dropdown' )
                            .hasClass( 'show' ) ) {

                            var $dropdown = $( this )
                                .siblings( '.header-nav-features-dropdown' );

                            $( '.header-nav-features-dropdown.show' )
                                .removeClass( 'show' );

                            $dropdown.addClass( 'show' );

                            $( document )
                                .off( 'click.header-nav-features-toggle' )
                                .on( 'click.header-nav-features-toggle', function ( e ) {
                                    if ( !$toggleParent.is( e.target ) && $toggleParent.has( e.target )
                                        .length === 0 ) {
                                        $( '.header-nav-features-dropdown.show' )
                                            .removeClass( 'show' );
                                    }
                                } );

                            if ( $( this )
                                .attr( 'data-focus' ) ) {
                                $( '#' + $( this )
                                        .attr( 'data-focus' ) )
                                    .focus();
                            }

                        } else {
                            $( this )
                                .siblings( '.header-nav-features-dropdown' )
                                .removeClass( 'show' );
                        }
                    } );

                // Hamburguer Menu
                var $hamburguerMenuBtn = $( '.hamburguer-btn:not(.side-panel-toggle)' ),
                    $hamburguerSideHeader = $( '#header.side-header, #header.side-header-overlay-full-screen' );

                $hamburguerMenuBtn.on( 'click', function () {
                    if ( $( this )
                        .attr( 'data-set-active' ) != 'false' ) {
                        $( this )
                            .toggleClass( 'active' );
                    }
                    $hamburguerSideHeader.toggleClass( 'side-header-hide' );
                    $html.toggleClass( 'side-header-hide' );

                    $window.trigger( 'resize' );
                } );

                $( '.hamburguer-close:not(.side-panel-toggle)' )
                    .on( 'click', function () {
                        $( '.hamburguer-btn:not(.hamburguer-btn-side-header-mobile-show)' )
                            .trigger( 'click' );
                    } );

                // Set Header Body Height when open mobile menu
                $( '.header-nav-main nav' )
                    .on( 'show.bs.collapse', function () {
                        $( this )
                            .removeClass( 'closed' );

                        // Add Mobile Menu Open Class
                        $( 'html' )
                            .addClass( 'mobile-menu-open' );

                        $( '.header-body' )
                            .animate( {
                                height: ( $( '.header-body' )
                                    .outerHeight() + $( '.header-nav-main nav' )
                                    .outerHeight( true ) ) + 10
                            } );

                        // Header Below Slider / Header Bottom Slider - Scroll to menu position
                        if ( $( '#header' )
                            .is( '.header-bottom-slider, .header-below-slider' ) && !$( 'html' )
                            .hasClass( 'sticky-header-active' ) ) {
                            self.scrollToTarget( $( '#header' ), 0 );
                        }
                    } );

                // Set Header Body Height when collapse mobile menu
                $( '.header-nav-main nav' )
                    .on( 'hide.bs.collapse', function () {
                        $( this )
                            .addClass( 'closed' );

                        // Remove Mobile Menu Opened Class
                        $( 'html' )
                            .removeClass( 'mobile-menu-open' );

                        $( '.header-body' )
                            .animate( {
                                height: ( $( '.header-body' )
                                    .outerHeight() - $( '.header-nav-main nav' )
                                    .outerHeight( true ) )
                            }, function () {
                                $( this )
                                    .height( 'auto' );
                            } );
                    } );

                // Header Effect Shrink - Adjust header body height on mobile
                $window.on( 'stickyHeader.activate', function () {
                    if ( $window.width() < 992 && $header.hasClass( 'header-effect-shrink' ) ) {
                        if ( $( '.header-btn-collapse-nav' )
                            .attr( 'aria-expanded' ) == 'true' ) {
                            $( '.header-body' )
                                .animate( {
                                    height: ( $( '.header-nav-main nav' )
                                        .outerHeight( true ) + theme.StickyHeader.options.stickyHeaderContainerHeight ) + ( ( $( '.header-nav-bar' )
                                            .get( 0 ) ) ? $( '.header-nav-bar' )
                                        .outerHeight() : 0 )
                                } );
                        }
                    }
                } );

                $window.on( 'stickyHeader.deactivate', function () {
                    if ( $window.width() < 992 && $header.hasClass( 'header-effect-shrink' ) ) {
                        if ( $( '.header-btn-collapse-nav' )
                            .attr( 'aria-expanded' ) == 'true' ) {
                            $( '.header-body' )
                                .animate( {
                                    height: headerBodyHeight + $( '.header-nav-main nav' )
                                        .outerHeight( true ) + 10
                                } );
                        }
                    }
                } );

                // Remove Open Class on Resize
                $window.on( 'resize.removeOpen', function () {
                    setTimeout( function () {
                        if ( $window.width() > 991 ) {
                            $header.find( '.dropdown.open' )
                                .removeClass( 'open' );
                        }
                    }, 100 );
                } );

                // Side Header - Change value of initial header body height
                $( document )
                    .ready( function () {
                        if ( $window.width() > 991 ) {
                            var flag = false;

                            $window.on( 'resize', function () {

                                $header.find( '.dropdown.open' )
                                    .removeClass( 'open' );

                                if ( $window.width() < 992 && flag == false ) {
                                    headerBodyHeight = $( '.header-body' )
                                        .outerHeight();
                                    flag = true;

                                    setTimeout( function () {
                                        flag = false;
                                    }, 500 );
                                }
                            } );
                        }
                    } );

                // Side Header - Set header height on mobile
                if ( $html.hasClass( 'side-header' ) ) {
                    if ( $window.width() < 992 ) {
                        $header.css( {
                            height: $( '.header-body .header-container' )
                                .outerHeight() + ( parseInt( $( '.header-body' )
                                    .css( 'border-top-width' ) ) + parseInt( $( '.header-body' )
                                    .css( 'border-bottom-width' ) ) )
                        } );
                    }

                    $( document )
                        .ready( function () {
                            $window.afterResize( function () {
                                if ( $window.width() < 992 ) {
                                    $header.css( {
                                        height: $( '.header-body .header-container' )
                                            .outerHeight() + ( parseInt( $( '.header-body' )
                                                .css( 'border-top-width' ) ) + parseInt( $( '.header-body' )
                                                .css( 'border-bottom-width' ) ) )
                                    } );
                                } else {
                                    $header.css( {
                                        height: ''
                                    } );
                                }
                            } );
                        } );
                }

                // Anchors Position
                $( '[data-hash]' )
                    .each( function () {

                        var target = $( this )
                            .attr( 'href' ),
                            offset = ( $( this )
                                .is( "[data-hash-offset]" ) ? $( this )
                                .data( 'hash-offset' ) : 0 );

                        if ( $( target )
                            .get( 0 ) ) {
                            $( this )
                                .on( 'click', function ( e ) {
                                    e.preventDefault();

                                    if ( !$( e.target )
                                        .is( 'i' ) ) {

                                        // Close Collapse if open
                                        $( this )
                                            .parents( '.collapse.show' )
                                            .collapse( 'hide' );

                                        // Close Side Header
                                        $hamburguerSideHeader.addClass( 'side-header-hide' );
                                        $html.addClass( 'side-header-hide' );

                                        $window.trigger( 'resize' );

                                        self.scrollToTarget( target, offset );

                                    }

                                    return;
                                } );
                        }

                    } );

                // Floating
                if ( $( '#header.header-floating-icons' )
                    .get( 0 ) ) {

                    $( '#header.header-floating-icons [data-hash]' )
                        .off()
                        .each( function () {

                            var target = $( this )
                                .attr( 'href' ),
                                offset = ( $( this )
                                    .is( "[data-hash-offset]" ) ? $( this )
                                    .data( 'hash-offset' ) : 0 );

                            if ( $( target )
                                .get( 0 ) ) {
                                $( this )
                                    .on( 'click', function ( e ) {
                                        e.preventDefault();

                                        $( 'html, body' )
                                            .animate( {
                                                scrollTop: $( target )
                                                    .offset()
                                                    .top - offset
                                            }, 600, 'easeOutQuad', function () {

                                            } );

                                        return;
                                    } );
                            }

                        } );

                }

                // Side Panel Toggle
                if ( $( '.side-panel-toggle' )
                    .get( 0 ) ) {
                    var init_html_class = $( 'html' )
                        .attr( 'class' );

                    $( '.side-panel-toggle' )
                        .on( 'click', function ( e ) {
                            var extra_class = $( this )
                                .data( 'extra-class' ),
                                delay = ( extra_class ) ? 100 : 0;

                            e.preventDefault();

                            if ( $( this )
                                .hasClass( 'active' ) ) {
                                $( 'html' )
                                    .removeClass( 'side-panel-open' );
                                $( '.hamburguer-btn.side-panel-toggle:not(.side-panel-close)' )
                                    .removeClass( 'active' );
                                return false;
                            }

                            if ( extra_class ) {
                                $( '.side-panel-wrapper' )
                                    .css( 'transition', 'none' );
                                $( 'html' )
                                    .removeClass()
                                    .addClass( init_html_class )
                                    .addClass( extra_class );
                            }

                            setTimeout( function () {
                                $( '.side-panel-wrapper' )
                                    .css( 'transition', '' );
                                $( 'html' )
                                    .toggleClass( 'side-panel-open' );
                            }, delay );
                        } );

                    $( document )
                        .on( 'click', function ( e ) {
                            if ( !$( e.target )
                                .closest( '.side-panel-wrapper' )
                                .get( 0 ) && !$( e.target )
                                .hasClass( 'side-panel-toggle' ) ) {
                                $( '.hamburguer-btn.side-panel-toggle:not(.side-panel-close)' )
                                    .removeClass( 'active' );
                                $( 'html' )
                                    .removeClass( 'side-panel-open' );
                            }
                        } );
                }

                return this;
            },

            scrollToTarget: function ( target, offset ) {
                var self = this;

                $( 'body' )
                    .addClass( 'scrolling' );

                $( 'html, body' )
                    .animate( {
                        scrollTop: $( target )
                            .offset()
                            .top - offset
                    }, self.options.scrollDelay, self.options.scrollAnimation, function () {
                        $( 'body' )
                            .removeClass( 'scrolling' );
                    } );

                return this;

            }

        }

    } );

} )
.apply( this, [ window.theme, jQuery ] );

// Sticky Header
( function ( theme, $ ) {

    theme = theme || {};

    var initialized = false;

    $.extend( theme, {

        StickyHeader: {

            defaults: {
                wrapper: $( '#header' ),
                headerBody: $( '#header .header-body' ),
                stickyEnabled: true,
                stickyEnableOnBoxed: true,
                stickyEnableOnMobile: true,
                stickyStartAt: 0,
                stickyStartAtElement: false,
                stickySetTop: 0,
                stickyEffect: '',
                stickyHeaderContainerHeight: false,
                stickyChangeLogo: false,
                stickyChangeLogoWrapper: true
            },

            initialize: function ( $wrapper, opts ) {
                if ( initialized ) {
                    return this;
                }

                initialized = true;
                this.$wrapper = ( $wrapper || this.defaults.wrapper );

                this
                    .setOptions( opts )
                    .build()
                    .events();

                return this;
            },

            setOptions: function ( opts ) {
                this.options = $.extend( true, {}, this.defaults, opts, theme.fn.getOptions( this.$wrapper.data( 'plugin-options' ) ) );

                return this;
            },

            build: function () {
                if ( !this.options.stickyEnableOnBoxed && $( 'html' )
                    .hasClass( 'boxed' ) || $( 'html' )
                    .hasClass( 'side-header-hamburguer-sidebar' ) || !this.options.stickyEnabled ) {
                    return this;
                }

                var self = this,
                    $html = $( 'html' ),
                    $window = $( window ),
                    sideHeader = $html.hasClass( 'side-header' ),
                    initialHeaderTopHeight = self.options.wrapper.find( '.header-top' )
                    .outerHeight(),
                    initialHeaderContainerHeight = self.options.wrapper.find( '.header-container' )
                    .outerHeight(),
                    minHeight;

                // HTML Classes
                $html.addClass( 'sticky-header-enabled' );

                if ( parseInt( self.options.stickySetTop ) < 0 ) {
                    $html.addClass( 'sticky-header-negative' );
                }

                // Set Start At
                if ( self.options.stickyStartAtElement ) {

                    var $stickyStartAtElement = $( self.options.stickyStartAtElement );

                    $( window )
                        .on( 'scroll resize', function () {
                            self.options.stickyStartAt = $stickyStartAtElement.offset()
                                .top;
                        } );

                    $( window )
                        .trigger( 'resize' );
                }

                // Define Min Height value
                if ( self.options.wrapper.find( '.header-top' )
                    .get( 0 ) ) {
                    minHeight = ( initialHeaderTopHeight + initialHeaderContainerHeight );
                } else {
                    minHeight = initialHeaderContainerHeight;
                }

                // Set Wrapper Min-Height
                if ( !sideHeader ) {
                    if ( !$( '.header-logo-sticky-change' )
                        .get( 0 ) ) {
                        self.options.wrapper.css( 'height', self.options.headerBody.outerHeight() );
                    } else {
                        $window.on( 'stickyChangeLogo.loaded', function () {
                            self.options.wrapper.css( 'height', self.options.headerBody.outerHeight() );
                        } );
                    }

                    if ( self.options.stickyEffect == 'shrink' ) {

                        // Prevent wrong visualization of header when reload on middle of page
                        $( document )
                            .ready( function () {
                                if ( $window.scrollTop() >= self.options.stickyStartAt ) {
                                    self.options.wrapper.find( '.header-container' )
                                        .on( 'transitionend webkitTransitionEnd oTransitionEnd', function () {
                                            self.options.headerBody.css( 'position', 'fixed' );
                                        } );
                                } else {
                                    self.options.headerBody.css( 'position', 'fixed' );
                                }
                            } );

                        self.options.wrapper.find( '.header-container' )
                            .css( 'height', initialHeaderContainerHeight );
                        self.options.wrapper.find( '.header-top' )
                            .css( 'height', initialHeaderTopHeight );
                    }
                }

                // Sticky Header Container Height
                if ( self.options.stickyHeaderContainerHeight ) {
                    self.options.wrapper.find( '.header-container' )
                        .css( 'height', self.options.wrapper.find( '.header-container' )
                            .outerHeight() );
                }

                // Boxed
                if ( $html.hasClass( 'boxed' ) && self.options.stickyEffect == 'shrink' ) {
                    if ( ( parseInt( self.options.stickyStartAt ) == 0 ) && $window.width() > 991 ) {
                        self.options.stickyStartAt = 30;
                    }

                    // Set Header Body Position Absolute
                    self.options.headerBody.css( 'position', 'absolute' );

                    // Set position absolute because top margin from boxed layout
                    $window.on( 'scroll', function () {
                        if ( $window.scrollTop() > $( '.body' )
                            .offset()
                            .top ) {
                            self.options.headerBody.css( {
                                'position': 'fixed',
                                'top': 0
                            } );
                        } else {
                            self.options.headerBody.css( {
                                'position': 'absolute',
                                'top': 0
                            } );
                        }
                    } );
                }

                // Check Sticky Header / Flags prevent multiple runs at same time
                var activate_flag = true,
                    deactivate_flag = false;

                self.checkStickyHeader = function () {
                    if ( $window.width() > 991 && $html.hasClass( 'side-header' ) ) {
                        $html.removeClass( 'sticky-header-active' );
                        activate_flag = true;
                        return;
                    }

                    if ( $window.scrollTop() >= parseInt( self.options.stickyStartAt ) ) {
                        if ( activate_flag ) {
                            self.activateStickyHeader();
                            activate_flag = false;
                            deactivate_flag = true;
                        }
                    } else {
                        if ( deactivate_flag ) {
                            self.deactivateStickyHeader();
                            deactivate_flag = false;
                            activate_flag = true;
                        }
                    }
                };

                // Activate Sticky Header
                self.activateStickyHeader = function () {

                    if ( $window.width() < 992 ) {
                        if ( !self.options.stickyEnableOnMobile ) {
                            self.deactivateStickyHeader();
                            return;
                        }
                    } else {
                        if ( sideHeader ) {
                            self.deactivateStickyHeader();
                            return;
                        }
                    }

                    $html.addClass( 'sticky-header-active' );

                    // Sticky Effect - Reveal
                    if ( self.options.stickyEffect == 'reveal' ) {

                        self.options.headerBody.css( 'top', '-' + self.options.stickyStartAt + 'px' );

                        self.options.headerBody.animate( {
                            top: self.options.stickySetTop
                        }, 400, function () {} );

                    }

                    // Sticky Effect - Shrink
                    if ( self.options.stickyEffect == 'shrink' ) {

                        // If Header Top
                        if ( self.options.wrapper.find( '.header-top' )
                            .get( 0 ) ) {
                            self.options.wrapper.find( '.header-top' )
                                .css( {
                                    height: 0,
                                    'min-height': 0,
                                    overflow: 'hidden'
                                } );
                        }

                        // Header Container
                        if ( self.options.stickyHeaderContainerHeight ) {
                            self.options.wrapper.find( '.header-container' )
                                .css( {
                                    height: self.options.stickyHeaderContainerHeight,
                                    'min-height': 0
                                } );
                        } else {
                            self.options.wrapper.find( '.header-container' )
                                .css( {
                                    height: ( initialHeaderContainerHeight / 3 ) * 2, // two third of container height
                                    'min-height': 0
                                } );

                            var y = initialHeaderContainerHeight - ( ( initialHeaderContainerHeight / 3 ) * 2 );
                            $( '.main' )
                                .css( {
                                    transform: 'translate3d(0, -' + y + 'px, 0)',
                                    transition: 'ease transform 300ms'
                                } );

                            if ( $html.hasClass( 'boxed' ) ) {
                                self.options.headerBody.css( 'position', 'fixed' );
                            }
                        }

                    }

                    self.options.headerBody.css( 'top', self.options.stickySetTop );

                    if ( self.options.stickyChangeLogo ) {
                        self.changeLogo( true );
                    }

                    // Set Elements Style
                    $( '[data-sticky-header-style]' )
                        .each( function () {
                            var $el = $( this ),
                                css = theme.fn.getOptions( $el.data( 'sticky-header-style-active' ) ),
                                opts = theme.fn.getOptions( $el.data( 'sticky-header-style' ) );

                            if ( $window.width() > opts.minResolution ) {
                                $el.css( css );
                            }
                        } );

                    $.event.trigger( {
                        type: 'stickyHeader.activate'
                    } );
                };

                // Deactivate Sticky Header
                self.deactivateStickyHeader = function () {

                    $html.removeClass( 'sticky-header-active' );

                    // Sticky Effect - Shrink
                    if ( self.options.stickyEffect == 'shrink' ) {

                        // Boxed Layout
                        if ( $html.hasClass( 'boxed' ) ) {

                            // Set Header Body Position Absolute
                            self.options.headerBody.css( 'position', 'absolute' );

                            if ( $window.scrollTop() > $( '.body' )
                                .offset()
                                .top ) {
                                // Set Header Body Position Fixed
                                self.options.headerBody.css( 'position', 'fixed' );
                            }

                        } else {
                            // Set Header Body Position Fixed
                            self.options.headerBody.css( 'position', 'fixed' );
                        }

                        // If Header Top
                        if ( self.options.wrapper.find( '.header-top' )
                            .get( 0 ) ) {
                            self.options.wrapper.find( '.header-top' )
                                .css( {
                                    height: initialHeaderTopHeight,
                                    overflow: 'visible'
                                } );
                        }

                        // Header Container
                        self.options.wrapper.find( '.header-container' )
                            .css( {
                                height: initialHeaderContainerHeight
                            } );

                    }

                    self.options.headerBody.css( 'top', 0 );

                    if ( self.options.stickyChangeLogo ) {
                        self.changeLogo( false );
                    }

                    // Set Elements Style
                    $( '[data-sticky-header-style]' )
                        .each( function () {
                            var $el = $( this ),
                                css = theme.fn.getOptions( $el.data( 'sticky-header-style-deactive' ) ),
                                opts = theme.fn.getOptions( $el.data( 'sticky-header-style' ) );

                            if ( $window.width() > opts.minResolution ) {
                                $el.css( css );
                            }
                        } );

                    $.event.trigger( {
                        type: 'stickyHeader.deactivate'
                    } );
                };

                // Always Sticky
                if ( parseInt( self.options.stickyStartAt ) <= 0 ) {
                    self.activateStickyHeader();
                }

                // Notice Top Bar
                if ( $( '.notice-top-bar' )
                    .get( 0 ) ) {
                    self.options.stickyStartAt = $( '.notice-top-bar' )
                        .outerHeight();
                }

                // Set Logo
                if ( self.options.stickyChangeLogo ) {

                    var $logoWrapper = self.options.wrapper.find( '.header-logo' ),
                        $logo = $logoWrapper.find( 'img' ),
                        logoWidth = $logo.attr( 'width' ),
                        logoHeight = $logo.attr( 'height' ),
                        logoSmallTop = parseInt( $logo.attr( 'data-sticky-top' ) ? $logo.attr( 'data-sticky-top' ) : 0 ),
                        logoSmallWidth = parseInt( $logo.attr( 'data-sticky-width' ) ? $logo.attr( 'data-sticky-width' ) : 'auto' ),
                        logoSmallHeight = parseInt( $logo.attr( 'data-sticky-height' ) ? $logo.attr( 'data-sticky-height' ) : 'auto' );

                    if ( self.options.stickyChangeLogoWrapper ) {
                        $logoWrapper.css( {
                            'width': $logo.outerWidth( true ),
                            'height': $logo.outerHeight( true )
                        } );
                    }

                    self.changeLogo = function ( activate ) {
                        if ( activate ) {

                            $logo.css( {
                                'top': logoSmallTop,
                                'width': logoSmallWidth,
                                'height': logoSmallHeight
                            } );

                        } else {

                            $logo.css( {
                                'top': 0,
                                'width': logoWidth,
                                'height': logoHeight
                            } );

                        }
                    };

                    $.event.trigger( {
                        type: 'stickyChangeLogo.loaded'
                    } );

                }

                // Side Header
                var headerBodyHeight,
                    flag = false;

                self.checkSideHeader = function () {
                    if ( $window.width() < 992 && flag == false ) {
                        headerBodyHeight = self.options.headerBody.height();
                        flag = true;
                    }

                    if ( self.options.stickyStartAt == 0 && sideHeader ) {
                        self.options.wrapper.css( 'min-height', 0 );
                    }

                    if ( self.options.stickyStartAt > 0 && sideHeader && $window.width() < 992 ) {
                        self.options.wrapper.css( 'min-height', headerBodyHeight );
                    }
                };

                return this;
            },

            events: function () {
                var self = this;

                if ( !this.options.stickyEnableOnBoxed && $( 'body' )
                    .hasClass( 'boxed' ) || $( 'html' )
                    .hasClass( 'side-header-hamburguer-sidebar' ) || !this.options.stickyEnabled ) {
                    return this;
                }

                if ( !self.options.alwaysStickyEnabled ) {
                    $( window )
                        .on( 'scroll resize', function () {
                            self.checkStickyHeader();
                        } );
                } else {
                    self.activateStickyHeader();
                }

                $( window )
                    .on( 'load resize', function () {
                        self.checkSideHeader();
                    } );

                return this;
            }

        }

    } );

} )
.apply( this, [ window.theme, jQuery ] );
